#!/usr/bin/python
# -*- coding:utf-8 -*-
# ----------------------------------------

# ----------------------------------------
# import modules
# ----------------------------------------
import os, sys, glob, re, fnmatch, datetime, numpy
import numpy as np
import scipy as sc


def L2130_raw(datestart, dateend, pathdata):
    datecurr = datestart
    data = {}
    avkeys = ['H2O', 'd18O', 'd2H', 'dexc', 'CP', 'OV', 'datetime', 'xsectime', 'WBT', 'DAS', 'CT']
    for k in avkeys:
        data[k] = []

    while datecurr <= dateend:

        if ((datecurr.day == 15 or datecurr.day == 16) and datecurr.month == 12):
            print('skip ' + str(datecurr))

        else:
            # read raw data
            L2130raw = read_data_L2130(datecurr, datecurr, pathdata)

            # read in calibration values for calibration runs of date (todo)
            if datecurr.day < 10:
                currday = '0' + str(datecurr.day)
            else:
                currday = str(datecurr.day)
            if datecurr.month < 10:
                currmonth = '0' + str(datecurr.month)
            else:
                currmonth = str(datecurr.month)
            datecurrstr = str(datecurr.year) + currmonth + currday

            # data manipulation for averaging
            #################
            # L2130
            #################

            # Length of arrays (if rsync while measuring the rows are sometimes not complete)
            minlen = 10e20
            for k in L2130raw.keys():
                if len(L2130raw[k]) < minlen:
                    minlen = len(L2130raw[k])

            L2130raw['datetime'] = []
            L2130raw['xsectime'] = []

            # construct time array
            for i in range(0, minlen):
                L2130raw['datetime'].append(datetime.datetime(int(L2130raw['DATE'][i][0:4]),
                                                              int(L2130raw['DATE'][i][5:7]),
                                                              int(L2130raw['DATE'][i][8:10]),
                                                              int(L2130raw['TIME'][i][0:2]),
                                                              int(L2130raw['TIME'][i][3:5]),
                                                              int(L2130raw['TIME'][i][6:8]),
                                                              int(L2130raw['TIME'][i][9:])))

            # Plotting arrays
            valves = np.array(L2130raw['ValveMask'][0:minlen])
            H2O = np.array(L2130raw['H2O'][0:minlen])
            d18O = np.array(L2130raw['Delta_18_16'][0:minlen])
            d2H = np.array(L2130raw['Delta_D_H'][0:minlen])
            dexc = np.array(L2130raw['Delta_D_H'][0:minlen]) - 8 * np.array(L2130raw['Delta_18_16'][0:minlen])
            CP = np.array(L2130raw['CavityPressure'][0:minlen])
            OV = np.array(L2130raw['OutletValve'][0:minlen])
            CT = np.array(L2130raw['CavityTemp'][0:minlen])
            DAS = np.array(L2130raw['DasTemp'][0:minlen])
            WBT = np.array(L2130raw['WarmBoxTemp'][0:minlen])

            # Flags for valve switch (SDM runs) or cavity pressure problems
            flag = np.zeros(np.shape(valves))
            for i in range(0, len(valves)):
                if valves[i] > 2.0:
                    starti = np.max([i - 60, 0])
                    endi = np.min([i + 180, len(valves)])
                    flag[starti:endi] = 1
                if (CP[i] > 50.2) or (CP[i] < 49.8):
                    starti = np.max([i - 30, 0])
                    endi = np.min([i + 30, len(CP)])
                    flag[starti:endi] = 1

            H2O[flag != 0] = np.nan
            d18O[flag != 0] = np.nan
            d2H[flag != 0] = np.nan
            dexc[flag != 0] = np.nan
            CP[flag != 0] = np.nan
            OV[flag != 0] = np.nan

            L2130rawflag = {}
            L2130rawflag['H2O'] = H2O
            L2130rawflag['d18O'] = d18O
            L2130rawflag['d2H'] = d2H
            L2130rawflag['dexc'] = dexc
            L2130rawflag['CP'] = CP
            L2130rawflag['CT'] = CT
            L2130rawflag['WBT'] = WBT
            L2130rawflag['DAS'] = DAS
            L2130rawflag['OV'] = OV
            L2130rawflag['datetime'] = L2130raw['datetime']
            L2130rawflag['xsectime'] = L2130raw['xsectime']

            for k in avkeys:
                for ii in range(len(L2130rawflag[k])):
                    data[k].append(L2130rawflag[k][ii])

        datecurr = datecurr + datetime.timedelta(days=1)

    for i in range(len(data['datetime'])):
        data['xsectime'].append((data['datetime'][i] - data['datetime'][0]).total_seconds())
    return data


#################################################
# Interpolate L2130 data to regular 1sec time steps
#################################################
def inter_reg1sectime(L2130high, keys):
    timestart = L2130high['datetime'][0] - datetime.timedelta(microseconds=L2130high['datetime'][0].microsecond)
    timeend = L2130high['datetime'][-1]
    timedeltasec = (timeend - timestart).total_seconds()
    xtime1sec = np.arange(0, round(timedeltasec), 1)  # array with 1sec data
    xraw = L2130high['xsectime']
    L2130low = {}
    L2130low['datetime'] = []
    for k in keys:
        L2130low[k] = sc.interp(xtime1sec, xraw, L2130high[k])
    for i in range(0, len(xtime1sec)):
        for k in keys:
            if L2130low[k][i] == '': L2130low[k][i] = np.nan
        L2130low['datetime'].append(timestart + datetime.timedelta(seconds=np.float(xtime1sec[i])))

    print('len(L2130low["datetime"]', len(L2130low['datetime']))

    L2130low['xsectime'] = xtime1sec
    return L2130low


#################################################
# Aggregate L2130 isotope data to time level with
# resolution dt
#################################################
def aggregate_L2130(L2130high, avkeys, tresolutions=[300, 3600]):
    mathigh = L2130high
    data_agg = []
    data_agg.append(mathigh)
    for res in tresolutions:
        matagg = dict(zip(avkeys, [0] * len(avkeys)))
        for k in avkeys:
            matagg[k] = []
            matagg[k + '_sd'] = []
        matagg['datetime'] = []
        matagg['xsectime'] = []

        istart = 0
        if res <= 3600:
            while ((int(mathigh['datetime'][istart].minute) % (res / 60)) != 0) or (
                    mathigh['datetime'][istart].minute != 0) or (istart < res):
                istart = istart + 1
            istop = len(mathigh['datetime']) - 1;
            while ((int(mathigh['datetime'][istop].minute) % (res / 60)) != 0) or (
                    mathigh['datetime'][istop].minute != 0) or ((istop + 1) > (len(mathigh['datetime']) - 1)):
                istop = istop - 1

        # res time averaging
        for i in range(istart, istop, res):
            for k in avkeys:
                a = mathigh[k][i - res + 1:i + 1]
                c = mathigh['H2O'][i - res + 1:i + 1]
                div = np.nansum(c)
                if div != 0:
                    if np.isnan(a).all():
                        # print('yes')
                        matagg[k].append(np.nan)
                        matagg[k + '_sd'].append(np.nan)
                    else:
                        msk = np.where(~np.isnan(a))
                        # print(float(res)/2.)
                        # print((len(msk[0])))
                        if len(msk[0]) > float(res) / 2.:
                            # print(msk)
                            matagg[k].append(np.nansum(a[msk[0]] * c[msk[0]]) / np.nansum(c[msk]))
                            matagg[k + '_sd'].append(np.nanstd(a[msk[0]]))
                        else:
                            matagg[k].append(np.nan)
                            matagg[k + '_sd'].append(np.nan)
                else:
                    matagg[k].append(np.nan)
                    matagg[k + '_sd'].append(np.nan)

            matagg['datetime'].append(mathigh['datetime'][i])
            matagg['xsectime'].append(mathigh['xsectime'][i])
        data_agg.append(matagg)

    return data_agg


#################################################
# Read L2130 raw data for time interval 
# datestarti to dateendi
#################################################
def read_data_L2130(datestarti, dateendi, pathdata):
    # Start and end time
    # -----------------------------------
    datecurr = datestarti
    datecurrstr = str(datecurr.year) + str(datecurr.month) + str(datecurr.day)
    datecurriso = datecurr.isoformat()
    dateendstr = str(dateendi.year) + str(dateendi.month) + str(dateendi.day)
    l = []
    while datecurr <= dateendi:
        # Paths of input data
        # -----------------------------------
        if datecurr.month < 10:
            month = '0' + str(datecurr.month)
        else:
            month = str(datecurr.month)
        if datecurr.day < 10:
            day = '0' + str(datecurr.day)
        else:
            day = str(datecurr.day)
        pathvaptot = pathdata + str(datecurr.year) + '/' + month + '/' + day + '/'
        files = selectL2130datafiles(datecurriso, pathvaptot, datestarti, dateendi)
        # Read in and correct isotope data
        # -------------------------------------------------------------------------------
        datecurriso = datecurr.isoformat()
        # date of day being processed
        da = {'y': datecurriso[0:4], 'm': datecurriso[5:7], 'd': datecurriso[8:10]}
        ff = getfilelist(pathvaptot)
        if os.path.exists(pathvaptot):
            print("Processing directory: ", pathvaptot)
        # Read in all data files for day datecurr
        for x in ff:
            # f=open(pathvaptot+"/"+x,"r")
            f = open(x, "r")

            header = f.readline().split()
            # l+=map(lambda z: dict(zip(header,z.split()[:2]+map(float,z.split()[2:]))),f.readlines())
            l += [dict(zip(header, z.split()[:2] + [float(x) for x in z.split()[2:]])) for z in f.readlines()]
            f.close()
        datecurr = datecurr + datetime.timedelta(days=1)
        datecurrstr = str(datecurr.year) + str(datecurr.month) + str(datecurr.day)
    # Make a dictionary out of the list of dictionary in l
    data = {}
    for k, v in [(key, d[key]) for d in l for key in d]:
        if k not in data:
            data[k] = [v]
        else:
            data[k].append(v)
    return data


#################################################
# Select L2130 data files
#################################################
def selectL2130datafiles(datecurriso, pathvaptot, datestart, dateend):
    if os.path.exists(pathvaptot):

        # open output file
        filelist = getfilelist(pathvaptot)
        fileyyyy = [int(os.path.basename(z)[9:13]) for z in filelist]
        filemm = [int(os.path.basename(z)[13:15]) for z in filelist]
        filedd = [int(os.path.basename(z)[15:17]) for z in filelist]
        fileHH = [int(os.path.basename(z)[18:20]) for z in filelist]
        fileMM = [int(os.path.basename(z)[20:22]) for z in filelist]
        filetime = []
        fileselect = []
        for i in range(0, len(filelist)):
            filetime.append(datetime.datetime(fileyyyy[i], filemm[i], filedd[i], fileHH[i], fileMM[i]))
        for i in range(0, len(filelist)):
            if filetime[i] >= datestart - datetime.timedelta(hours=1) and filetime[i] <= dateend:
                fileselect.append(filelist[i])
    else:
        fileselect = []
        print("L2130 data folder not found %s" % (pathvaptot))
    return fileselect


#################################################
# List L2130 files for 1  day
#################################################
def getfilelist(pathvaptot):
    if os.path.exists(pathvaptot):
        ff = []
        for root, dirs, files in os.walk(pathvaptot):
            for file in files:
                f = os.path.join(root, file)
                if f[-16:] == "DataLog_User.dat":
                    ff.append(f)
        ff.sort(key=lambda x: x[-len('HBDS2190-20170124-225106Z-DataLog_User.dat'):])
        return ff
    else:
        ff = []
        return ff


#################################################
# Read in SWI8 raw data
#################################################
def P22_raw(path, datestart, dateend, instr, CP_reg=35.0):
    '''
    path (string): path to file
    datestart (datetime): first date to read in
    dateend (datetime: last date to read in
    instr (string): instrument used (SWI8-ps or SWI8-sb)
    CP_reg (float): cavity pressure [torr] of instrument (35 torr for L2120, 50 torr for L2130)
    '''

    # read raw data
    L2130raw = read_data_p22(path, datestart, dateend)

    # data manipulation for averaging
    #################
    # SWI raw data
    #################

    # Length of arrays (if rsync while measuring the rows are sometimes not complete)
    minlen = int(10e20)
    for k in L2130raw.keys():
        if len(L2130raw[k]) < minlen:
            minlen = len(L2130raw[k])

    L2130raw['datetime'] = []

    # construct time array
    for i in range(0, minlen):
        # print(L2130raw['DATE'][i])
        date = L2130raw['DATE'][i]
        date_format = re.match(r'([0-9]{4})-([0-9]{2})-([0-9]{2})', date)
        if date_format is not None:
            L2130raw['datetime'].append(datetime.datetime(int(L2130raw['DATE'][i][0:4]),
                                                          int(L2130raw['DATE'][i][5:7]),
                                                          int(L2130raw['DATE'][i][8:10]),
                                                          int(L2130raw['TIME'][i][0:2]),
                                                          int(L2130raw['TIME'][i][3:5]),
                                                          int(L2130raw['TIME'][i][6:8]),
                                                          int(L2130raw['TIME'][i][9:])))

        else:
            date_format = re.match(r'([0-9]{2})\.([0-9]{2})\.([0-9]{4})', date)
            if date_format is not None:
                L2130raw['datetime'].append(datetime.datetime(int(L2130raw['DATE'][i][6:10]),
                                                              int(L2130raw['DATE'][i][3:5]),
                                                              int(L2130raw['DATE'][i][0:2]),
                                                              int(L2130raw['TIME'][i][0:2]),
                                                              int(L2130raw['TIME'][i][3:5]),
                                                              int(L2130raw['TIME'][i][6:8]),
                                                              int(L2130raw['TIME'][i][9:])))
        if not date_format:
            assert False, f"Date format not recognised, {date}"

    # Plotting arrays
    valves = np.array(L2130raw['ValveMask'][0:minlen])
    H2O = np.array(L2130raw['H2O'][0:minlen])
    d18O = np.array(L2130raw['Delta_18_16'][0:minlen])
    d2H = np.array(L2130raw['Delta_D_H'][0:minlen])
    dexc = np.array(L2130raw['Delta_D_H'][0:minlen]) - 8 * np.array(L2130raw['Delta_18_16'][0:minlen])
    CP = np.array(L2130raw['CavityPressure'][0:minlen])
    OV = np.array(L2130raw['OutletValve'][0:minlen])
    CT = np.array(L2130raw['CavityTemp'][0:minlen])
    DAS = np.array(L2130raw['DasTemp'][0:minlen])
    WBT = np.array(L2130raw['WarmBoxTemp'][0:minlen])

    # Flags for valve switch or cavity environment problems
    flag = np.zeros(np.shape(valves))
    for i in range(0, len(valves)):
        if valves[i] > 2.0:
            starti = np.max([i - 60, 0])
            endi = np.min([i + 180, len(valves)])
            flag[starti:endi] = 1
        if (CP[i] > CP_reg + 0.2) or (CP[i] < CP_reg - 0.2):
            starti = np.max([i - 30, 0])
            endi = np.min([i + 30, len(CP)])
            flag[starti:endi] = 1
        if (CT[i] > 80.02) or (CT[i] < 79.98):
            starti = np.max([i - 30, 0])
            endi = np.min([i + 30, len(CT)])
            flag[starti:endi] = 1

    H2O[flag != 0] = np.nan
    d18O[flag != 0] = np.nan
    d2H[flag != 0] = np.nan
    dexc[flag != 0] = np.nan
    CP[flag != 0] = np.nan
    OV[flag != 0] = np.nan

    L2130rawflag = {}
    L2130rawflag['H2O'] = H2O
    L2130rawflag['d18O'] = d18O
    L2130rawflag['d2H'] = d2H
    L2130rawflag['dexc'] = dexc
    L2130rawflag['CP'] = CP
    L2130rawflag['CT'] = CT
    L2130rawflag['WBT'] = WBT
    L2130rawflag['DAS'] = DAS
    L2130rawflag['OV'] = OV
    L2130rawflag['valves'] = valves
    L2130rawflag['datetime'] = L2130raw['datetime']
    L2130rawflag['xsectime'] = []

    for i in range(len(L2130rawflag['datetime'])):
        L2130rawflag['xsectime'].append((L2130rawflag['datetime'][i] - L2130rawflag['datetime'][0]).total_seconds())

    return L2130rawflag


#################################################
# Read p22 raw data for time interval 
# datestart to dateend
#################################################
def read_data_p22(pathvap, datestart, dateend):
    l = []

    # Paths of input data
    # -----------------------------------
    pathvaptot = pathvap
    # Read in and correct isotope data
    # -------------------------------------------------------------------------------
    ff = getfilelist(pathvaptot)
    if os.path.exists(pathvaptot):
        print("Processing directory: ", pathvaptot)
    # Read in all data files for day datecurr
    for x in ff:
        filename = os.path.basename(x)
        filedate = datetime.datetime(int(filename[9:13]), int(filename[13:15]), int(filename[15:17]))
        # print(filedate)
        if filedate <= dateend and filedate >= datestart:
            print('loading file: ' + x)
            f = open(x, "rb")
            # header=f.readline().split()
            header = []
            binary_header = f.readline().split()
            for bin_h in binary_header:
                h = bin_h.decode('ascii')
                header.append(h)

            # l+=map(lambda z: dict(zip(header,z.split()[:2]+map(float,z.split()[2:]))),f.readlines())
            line_number = 0
            for binary_z in f.readlines():
                line_number += 1
                try:
                    z = binary_z.decode('ascii')
                except ValueError:
                    print('It failed reading file', filename, 'in line', line_number)
                    continue

                new = dict(zip(header, z.split()[:2] + [float(x) for x in z.split()[2:]]))
                l.append(new)

            # l+=[dict(zip(header,z.split()[:2]+[float(x) for x in z.split()[2:]])) for z in f.readlines()]
            f.close()

    # Make a dictionary out of the list of dictionary in l
    data = {}

    for k, v in [(key, d[key]) for d in l for key in d]:
        # print(k)
        if v == '': v = np.nan
        if k not in ['DATE', 'TIME']:
            if k not in data:
                data[k] = [np.float(v)]
            else:
                data[k].append(np.float(v))
        else:
            if k not in data:
                data[k] = [v]
            else:
                data[k].append(v)
    return data


def read_std(path, start, end, stdn):
    import numpy as np
    from datetime import datetime, timedelta

    """

    loading calibration run for SWI measurements

    path (string): Path to ship-data 
    start (datetime): start of chosen time interval
    end (datetime): end of chosen time interval
    stdn (array of strings): output calibration standards
    
    """

    #   open csv-file

    f = open(path, 'r')  # We need to open the file

    # Loop over lines and extract variables of interest

    picarro = []
    header_line = next(f)  # reading header
    header = header_line.split(',')
    print(header)

    for line in f:  # loop through lines of file and creat a list of dictionaries (shipmet)
        line = line.strip()
        columns = line.split(',')
        picarro += [dict(zip(header, columns))]
    f.close()

    #   compile dictionary from list

    l2130 = {}

    for k, v in [(key, d[key]) for d in picarro for key in d]:
        if v == '': v = np.nan
        if k not in ['time_UTC', 'std\n']:
            if k not in l2130:
                l2130[k] = [np.float(v)]
            else:
                l2130[k].append(np.float(v))
        else:
            if k not in l2130:
                l2130[k] = [v]
            else:
                l2130[k].append(v)

    l2130['datetime'] = []
    for i in range(len(l2130['time_UTC'])):  # change format of date (into float)
        a = l2130['time_UTC'][i].split('T')
        l2130['datetime'].append(datetime(int(a[0][0:4]), int(a[0][5:7]), int(a[0][8:10]),
                                          int(a[1][0:2]), int(a[1][3:5]), int(a[1][6:8])))
    if 'std\n' in l2130.keys(): l2130['std'] = l2130.pop('std\n')
    stds = {}
    kk = ['datetime', 'd2H', 'd18O', 'H2O', 'd2H_sd', 'd18O_sd', 'H2O_sd']
    for n in stdn:
        stds[n] = {}
        c = 0
        for k in kk:
            stds[n][k] = []
            for i in range(len(l2130['datetime'])):
                if l2130['std'][i] == n:
                    c = 1
                    if l2130['datetime'][i] >= start and l2130['datetime'][i] <= end:
                        stds[n][k].append(l2130[k][i])
        if c == 0: print(n + ' is not a valid calibration standard name for this data set')

    return stds


def load_flag(path, instr):
    f = open(path + '.csv', 'r')

    # Loop over lines and extract variables of interest

    shipmet = []
    header_line = next(f)  # reading header
    header = header_line.split(',')
    header[-1] = header[-1].rstrip()
    # print(header_line)
    print(header)

    for line in f:  # loop through lines of file and create a list of dictionaries (shipmet)
        line = line.strip()
        columns = line.split(',')

        shipmet += [dict(zip(header, columns))]

    f.close()

    #   compile dictionary from list

    meteo = {}
    # meteo['datetime']=[]
    for k, v in [(key, d[key]) for d in shipmet for key in d]:
        if v == '': v = 'nan'
        if k in ['datetime', 'datetime_start', 'datetime_stop']:
            if k not in meteo:
                meteo[k] = [v]
            else:
                meteo[k].append(v)
        else:
            # print(k)
            if k not in meteo:
                meteo[k] = [float(v)]
            else:
                meteo[k].append(float(v))

    for i in range(len(meteo['datetime_start'])):  # change format of date (into float)
        a = meteo['datetime_start'][i]
        # print(a)
        meteo['datetime_start'][i] = datetime.datetime(int(a[0:4]), int(a[5:7]), int(a[8:10]), int(a[11:13]),
                                                       int(a[14:16]), int(a[17:19]))
        a = meteo['datetime_stop'][i]
        # print(a)
        meteo['datetime_stop'][i] = datetime.datetime(int(a[0:4]), int(a[5:7]), int(a[8:10]), int(a[11:13]),
                                                      int(a[14:16]), int(a[17:19]))

    return meteo
